@@ -341,11 +341,15 @@ connection_send_keep_alive (LmConnection *connection)
 static void
 connection_start_keep_alive (LmConnection *connection)
 {
-	/* try using TCP keepalives if possible */
-	if ((connection->keep_alive_rate > 0) &&
-		lm_socket_set_keepalive (connection->socket,
-			connection->keep_alive_rate)) {
-		return;
+        /* try using TCP keepalives if possible */
+        if ((connection->keep_alive_rate > 0) &&
+            lm_socket_set_keepalive (connection->socket,
+                                     connection->keep_alive_rate)) {
+#ifdef ONLY_TCP_KEEP_ALIVE
+                /* Many NAT firewalls seems to not handle this correctly and 
+                 * will disconnect the clients */
+                return;
+#endif /* ONLY_TCP_KEEP_ALIVE */
 	}
 
 	if (connection->keep_alive_source) {
@@ -1540,9 +1544,25 @@ lm_connection_authenticate_and_block (LmConnection  *connection,
 	return FALSE;
 }
 
+/**
+ * lm_connection_get_keep_alive_rate:
+ * @connection: an #LmConnection 
+ *
+ * Get the keep alive rate, in seconds. Zero is returned if no keep alive rate has been set.
+ *
+ * Since 1.3.5
+ **/
+guint
+lm_connection_get_keep_alive_rate (LmConnection *connection)
+{
+	g_return_val_if_fail (connection != NULL, 0);
+
+	return connection->keep_alive_rate;
+}
+
 /**
  * lm_connection_set_keep_alive_rate:
- * @connection: #LmConnection to check if it is open.
+ * @connection: an #LmConnection
  * @rate: Number of seconds between keep alive packages are sent.
  * 
  * Set the keep alive rate, in seconds. Set to 0 to prevent keep alive messages to be sent.
